@@ -25,90 +25,135 @@
 #ifndef GWORKSPACE_H
 #define GWORKSPACE_H
 
+#include <Foundation/Foundation.h>
 #include <AppKit/NSApplication.h>
-  #ifdef GNUSTEP 
 #include "GWProtocol.h"
-  #else
-#include <GWorkspace/GWProtocol.h>
-  #endif
 
 #define NOEDIT 0
 #define NOXTERM 1
 
-@class NSString;
-@class NSArray;
-@class NSMutableArray;
-@class NSMutableDictionary;
-@class NSNotification;
-@class NSTimer;
-@class NSFileManager;
 @class NSWorkspace;
-@class ViewersWindow;
-@class InspectorsController;
+@class FSNodeRep;
+@class GWViewersManager;
+@class GWDesktopManager;
+@class Finder;
+@class Inspector;
+@class GWViewer;
 @class PrefController;
-@class FinderController;
-@class AppsViewer;
 @class Fiend;
-@class Recycler;
 @class History;
-@class DesktopWindow;
-@class DesktopView;
 @class TShelfWin;
-@class FileOperation;
 @class OpenWithController;
 @class RunExternalController;
+@class StartAppWin;
 
-@interface GWorkspace : NSObject <GWProtocol>
-{
-	NSString *defEditor, *defXterm, *defXtermArgs;
-	
-  NSMutableArray *operations;
-  int oprefnum;
-  BOOL showFileOpStatus;
+@protocol	FSWClientProtocol
+
+- (void)watchedPathDidChange:(NSData *)dirinfo;
+
+@end
+
+
+@protocol	FSWatcherProtocol
+
+- (oneway void)registerClient:(id <FSWClientProtocol>)client;
+
+- (oneway void)unregisterClient:(id <FSWClientProtocol>)client;
+
+- (oneway void)client:(id <FSWClientProtocol>)client
+                          addWatcherForPath:(NSString *)path;
+
+- (oneway void)client:(id <FSWClientProtocol>)client
+                          removeWatcherForPath:(NSString *)path;
+
+@end
+
+
+@protocol	RecyclerAppProtocol
+
+- (oneway void)emptyTrash:(id)sender;
+
+@end
+
+
+@protocol	OperationProtocol
+
+- (oneway void)performOperation:(NSData *)opinfo;
+
+- (oneway void)setFilenamesCutted:(BOOL)value;
+
+- (BOOL)filenamesWasCutted;
+
+@end
+
+
+@protocol	DDBdProtocol
+
+- (BOOL)dbactive;
+
+- (oneway void)insertPath:(NSString *)path;
+
+- (oneway void)removePath:(NSString *)path;
+
+- (NSString *)annotationsForPath:(NSString *)path;
+
+- (oneway void)setAnnotations:(NSString *)annotations
+                      forPath:(NSString *)path;
+
+@end
+
+
+@interface GWorkspace : NSObject <GWProtocol, FSWClientProtocol>
+{	
+  FSNodeRep *fsnodeRep;
   
-  InspectorsController *inspController;
 	NSArray *selectedPaths;
+
+  id fswatcher;
+  BOOL fswnotifications;
 	
-  AppsViewer *appsViewer;
-  FinderController *finder;
+  id operationsApp;
+  id recyclerApp;
+  
+  id ddbd;
+  
   PrefController *prefController;
   Fiend *fiend;
+  
   History *history;
-	
-  ViewersWindow *rootViewer, *currentViewer;	
-  NSMutableArray *viewers;
-  NSMutableArray *viewersSearchPaths;
-	NSMutableArray *viewersTemplates;
-
+	int maxHistoryCache;
+    
+  GWViewersManager *vwrsManager;
+  GWDesktopManager *dtopManager;  
+  Inspector *inspector;
+  Finder *finder;
+  
   BOOL animateChdir;
-  BOOL animateLaunck;
   BOOL animateSlideBack;
 
   BOOL contestualMenu;
 
   BOOL dontWarnOnQuit;
-
-  DesktopWindow *desktopWindow;
+  BOOL terminating;
   
   TShelfWin *tshelfWin;
-  NSImage *tshelfBackground;
   NSString *tshelfPBDir;
   int tshelfPBFileNum;
-  
-	Recycler *recycler;
-	NSString *trashPath;
-    
+      
   OpenWithController *openWithController;
   RunExternalController *runExtController;
-    
-  BOOL usesThumbnails;
-	      
+  
+  StartAppWin *startAppWin;
+    	      
   int shelfCellsWidth;
-        
+
+	NSString *defEditor;
+  NSString *defXterm;
+  NSString *defXtermArgs;
+  BOOL teminalService;
+          
   NSFileManager *fm;
   NSWorkspace *ws;
-  
-  BOOL starting;
 }
 
 + (void)registerForServices;
@@ -123,17 +168,15 @@
 
 - (NSString *)defXtermArgs;
 
+- (GWViewersManager *)viewersManager;
+
+- (GWDesktopManager *)desktopManager;
+
 - (History *)historyWindow;
 
-- (id)desktopView;	
-
-- (void)showHideDesktop:(BOOL)active;
-
 - (NSImage *)tshelfBackground;	
 
-- (void)makeTshelfBackground;
-
-- (NSColor *)tshelfBackColor;	
+- (void)tshelfBackgroundDidChange;
 
 - (NSString *)tshelfPBDir;
 
@@ -141,11 +184,14 @@
 
 - (id)rootViewer;
 
-- (ViewersWindow *)viewerRootedAtPath:(NSString *)vpath;
+- (void)newViewerAtPath:(NSString *)path;
 
-- (void)changeDefaultEditor:(NSString *)editor;
+- (void)changeDefaultEditor:(NSNotification *)notif;
 
-- (void)changeDefaultXTerm:(NSString *)xterm arguments:(NSString *)args;
+- (void)changeDefaultXTerm:(NSString *)xterm 
+                 arguments:(NSString *)args;
+             
+- (void)setUseTerminalService:(BOOL)value;             
                              
 - (void)updateDefaults;
 					 
@@ -161,74 +207,89 @@
 
 - (void)setShelfCellsWidth:(int)w; 
 
-- (void)createRecycler;
-
 - (void)createTabbedShelf;
 
-- (void)makeViewersTemplates;
-
-- (void)addViewer:(id)vwr withBundlePath:(NSString *)bpath;
-
-- (void)removeViewerWithBundlePath:(NSString *)bpath;
-
-- (NSMutableArray *)bundlesWithExtension:(NSString *)extension 
-											       inDirectory:(NSString *)dirpath;
-
-- (NSArray *)viewersPaths;
-
 - (void)checkViewersAfterHidingOfPaths:(NSArray *)paths;
 
-- (void)viewerHasClosed:(id)sender;
-
-- (void)setCurrentViewer:(ViewersWindow *)viewer;
-
 - (void)iconAnimationChanged:(NSNotification *)notif;
 
-- (BOOL)showFileOpStatus;
-
-- (void)setShowFileOpStatus:(BOOL)value;
-
 - (void)fileSystemWillChange:(NSNotification *)notif;
 
 - (void)fileSystemDidChange:(NSNotification *)notif;
-           
-- (void)watcherNotification:(NSNotification *)notification;           
-           
+                      
 - (void)setSelectedPaths:(NSArray *)paths;
 
 - (void)resetSelectedPaths;
 
-- (void)setSelectedPaths:(NSArray *)paths fromDesktopView:(DesktopView *)view;
-
-- (void)setSelectedPaths:(NSArray *)paths 
-         fromDesktopView:(DesktopView *)view
-            animateImage:(NSImage *)image 
-         startingAtPoint:(NSPoint)startp;
-
 - (NSArray *)selectedPaths;
 
 - (void)showPasteboardData:(NSData *)data 
                     ofType:(NSString *)type
                   typeIcon:(NSImage *)icon;
 
-- (void)closeInspectors;
-
-- (void)newObjectAtPath:(NSString *)basePath isDirectory:(BOOL)directory;
+- (void)newObjectAtPath:(NSString *)basePath 
+            isDirectory:(BOOL)directory;
 
 - (void)duplicateFiles;
 
 - (void)deleteFiles;
 
+- (void)moveToTrash;
+
 - (BOOL)verifyFileAtPath:(NSString *)path;
 
 - (void)setUsesThumbnails:(BOOL)value;
 
 - (void)thumbnailsDidChange:(NSNotification *)notif;
 
+- (void)applicationForExtensionsDidChange:(NSNotification *)notif;
+
+- (int)maxHistoryCache;
+
+- (void)setMaxHistoryCache:(int)value;
+
+- (void)connectFSWatcher;
+
+- (void)fswatcherConnectionDidDie:(NSNotification *)notif;
+
+- (void)connectRecycler;
+
+- (void)recyclerConnectionDidDie:(NSNotification *)notif;
+
+- (void)connectOperation;
+
+- (void)operationConnectionDidDie:(NSNotification *)notif;
+
+- (void)connectDDBd;
+
+- (void)ddbdConnectionDidDie:(NSNotification *)notif;
+
+- (BOOL)ddbdactive;
+
+- (void)ddbdInsertPath:(NSString *)path;
+
+- (void)ddbdRemovePath:(NSString *)path;
+
+- (NSString *)ddbdGetAnnotationsForPath:(NSString *)path;
+
+- (void)ddbdSetAnnotations:(NSString *)annotations
+                   forPath:(NSString *)path;
+
 - (id)connectApplication:(NSString *)appName;
 
+
+//
+// NSServicesRequests protocol
+//
 - (id)validRequestorForSendType:(NSString *)sendType
                      returnType:(NSString *)returnType;
+
+- (BOOL)readSelectionFromPasteboard:(NSPasteboard *)pboard;
+
+
+- (BOOL)writeSelectionToPasteboard:(NSPasteboard *)pboard
+                             types:(NSArray *)types;
+                             
 										 
 //
 // Menu Operations
@@ -251,11 +312,11 @@
 
 - (void)showToolsInspector:(id)sender;
 
-- (void)showPermissionsInspector:(id)sender;
+- (void)showAnnotationsInspector:(id)sender;
 
-- (void)showApps:(id)sender;
+- (void)showDesktop:(id)sender;
 
-- (void)showFileOps:(id)sender;
+- (void)showRecycler:(id)sender;
 
 - (void)showFinder:(id)sender;
 
@@ -287,25 +348,37 @@
 
 - (void)runCommand:(id)sender;
 
-- (void)startXTerm:(id)sender;
+- (void)checkRemovableMedia:(id)sender;
 
 - (void)emptyRecycler:(id)sender;
 
-- (void)putAway:(id)sender;
 
-#ifndef GNUSTEP
-- (void)terminate:(id)sender;
-#endif
+//
+// DesktopApplication protocol
+//
+- (void)selectionChanged:(NSArray *)newsel;
 
-@end
+- (void)openSelectionInNewViewer:(BOOL)newv;
 
-@interface GWorkspace (FileOperations)
+- (void)openSelectionWithApp:(id)sender;
 
-- (int)fileOperationRef;
+- (void)performFileOperation:(NSDictionary *)opinfo;
 
-- (FileOperation *)fileOpWithRef:(int)ref;
+- (void)lsfolderDragOperation:(NSData *)opinfo
+              concludedAtPath:(NSString *)path;
+                          
+- (void)concludeRemoteFilesDragOperation:(NSData *)opinfo
+                             atLocalPath:(NSString *)localPath;
 
-- (void)endOfFileOperation:(FileOperation *)op;
+// - (void)addWatcherForPath:(NSString *)path; // already in GWProtocol
+
+// - (void)removeWatcherForPath:(NSString *)path; // already in GWProtocol
+
+// - (NSString *)trashPath; // already in GWProtocol
+
+- (id)workspaceApplication;
+
+- (BOOL)terminating;
 
 @end
 
