@@ -25,35 +25,32 @@
 #ifndef TSHELF_ICON_H
 #define TSHELF_ICON_H
 
+#include <Foundation/Foundation.h>
 #include <AppKit/NSView.h>
 
 #define ONICON(p, s1, s2) ([self mouse: (p) \
 inRect: NSMakeRect(((int)(s1).width - (int)(s2).width) >> 1,\
 ((int)(s1).height - (int)(s2).height) >> 1, 48, 48)])
 
-@class NSString;
-@class NSArray;
-@class NSMutableArray;
-@class NSNotification;
 @class NSEvent;
 @class NSPasteboard;
 @class NSTextField;
 @class NSImage;
-@class NSFileManager;
 @class NSWorkspace;
 @class TShelfIconsView;
+@class FSNode;
+@class FSNodeRep;
 @class GWorkspace;
 
 @interface TShelfIcon : NSView
 {
   NSMutableArray *paths;
-  NSString *fullPath;
   NSString *name;
 	NSString *hostname;
+  FSNode *node;
   BOOL singlepath;
 	BOOL isRootIcon;	
 	BOOL isPakage;	
-  NSString *type;
   BOOL isSelect;
   BOOL locked;
   
@@ -65,12 +62,14 @@ inRect: NSMakeRect(((int)(s1).width - (int)(s2).width) >> 1,\
 
   TShelfIconsView *tview;
   
+  FSNodeRep *fsnodeRep;
   NSFileManager *fm;
 	NSWorkspace *ws;
   GWorkspace *gw;
   
   int dragdelay;
   BOOL isDragTarget;
+  BOOL forceCopy;
   BOOL onSelf;
 }
 
@@ -107,14 +106,8 @@ inRect: NSMakeRect(((int)(s1).width - (int)(s2).width) >> 1,\
 
 - (NSTextField *)myLabel;
 
-- (NSString *)type;
-
 - (NSArray *)paths;
 
-- (NSString *)name;
-
-- (NSString *)hostname;
-
 - (BOOL)isSinglePath;
 
 - (BOOL)isSelect;
@@ -123,10 +116,6 @@ inRect: NSMakeRect(((int)(s1).width - (int)(s2).width) >> 1,\
 
 - (BOOL)isLocked;
 
-- (BOOL)isRootIcon;
-
-- (BOOL)isPakage;
-
 @end
 
 @interface TShelfIcon (DraggingSource)
